#!/bin/bash

set -u 
set -e

# parses fdisk output to extract partition offset
# NB: It's responsibility of the caller to supply a standard input meeting this script's expectations, ie. being compatible with the output generated by our version: fdisk v2.12r
# M.B.Caminati

get_leftmost_number ()
{
# extract leftmost number from a string, ie 
# get_leftmost_number asdfsad53fsda>e56yre67 
# gives 53

OUTPUT="$(expr match "$1" '\([^0-9]*[0-9]*\)')"
OUTPUT="${OUTPUT##*[^0-9]}"
echo "$OUTPUT"
}

get_rightmost_number ()
{
OUTPUT="$(expr match "$1" '\(.*[0-9]\)')"
OUTPUT="${OUTPUT##*[^0-9]}"
echo "$OUTPUT"
}

isnumber ()
{
# Test if the first (supposed the only, too) argument is a number
if [ "$1" -eq "$1" 2> /dev/null ]
then
 return 0
else
 return 64
fi
}

PRIMARY_PARTITION_NUMBER="$1"
if [[ ($PRIMARY_PARTITION_NUMBER -ne 1) && ($PRIMARY_PARTITION_NUMBER -ne 2) && ($PRIMARY_PARTITION_NUMBER -ne 3) && ($PRIMARY_PARTITION_NUMBER -ne 4) ]]
then exit 65
fi

PRINTOUT="$(cat | tr '[A-Z]' '[a-z]')"

# Now we extract sector size from the printout of fdisk.
# If we don't get a positive number, it means fdisk didn't print out a partition table, so probably the VDI image file supplied is not all right.
# This way we accomplish a twofold goal in one step: checking goodness of input and possibly getting the sector size (in bytes)
SECTOR_SIZE="$( get_rightmost_number "$(echo "$PRINTOUT" |grep -i "^Units =")")"

if  isnumber "$SECTOR_SIZE" && [[ "$SECTOR_SIZE" -ge 1 ]] 
then [ 1 -eq 1 ]
else
 exit 66
fi

#PARTITION_TABLE_ROW=$(echo "$PRINTOUT" | tail --lines=4 | head --lines="$PRIMARY_PARTITION_NUMBER" | tail --lines=1)
PARTITION_TABLE_HEADER_ROW_NUMBER="$(echo "$PRINTOUT"|sed -n -e '/.*device.*boot.*start.*end.*/=' )"
PARTITION_TABLE_ROW_NUMBER="$(( $PARTITION_TABLE_HEADER_ROW_NUMBER + $PRIMARY_PARTITION_NUMBER ))"
PARTITION_TABLE_ROW="$(echo "$PRINTOUT" |sed -n -e ''$PARTITION_TABLE_ROW_NUMBER' p')"

OFFSET_IN_SECTORS="$(get_leftmost_number "${PARTITION_TABLE_ROW#*\ \ }")"
# This particular parsing is based on the empirical observation that there are at least two spaces between the end of the Device name and the possible '*' indicating the bootable flag of a partition in each entry of the partition table printing by fdisk.
# So a necessary condition for this parsing to be sensible is to avoid having more than one space in the filename of the loop device. (see remarks there and in vdi_mount)

echo $(($OFFSET_IN_SECTORS * $SECTOR_SIZE))
exit 0
