#!/bin/bash
# Copyright 2009 Marco B. Caminati.
# Released under GNU GPL.
set -u
# throws an error on any attempt to use unset variables (eg non-existent command line parameters)

set -e
# terminates at first error (so to enforce, in particular, the previous set -u directive)
 
LOSETUP_MAGIC_STRING="Loop device is "

SCRIPT_DIR="$(dirname "$(readlink -f "$0")")"
export SCRIPT_DIR

echo "$1" "$2" "$3" > /dev/null
# A lazyugly way to abort in case of insufficient numbers of command line arguments.

# Common header end

# This scripts takes a path to a non-dynamic (fixed) dvi image file, a mountpoint and a natural number n \in [1-4]
# It mounts the n-th PRIMARY partition contained in the dvi at the mountpoint.
# This is done in four steps:
# 1) Finding the offset within the dvi the mbr resides at
# 2) Pointing a loop device file to that offset, thus creating a virtual block device mimicking the hard disk contained in the dvi
# 3) Executing a sfdisk on the virtual block device to work out the offset within it of the partition specified by n.
# 4) Finally mounting the partition.

# Steps 1-2 are accomplished by $VDI_LOOP_SCRIPT, which needs vditool by SUN to be installed
# Step 3 is accomplished here with the help of $PARTTABLE_PARSER_SCRIPT
# Note that a total of two loop devices are used, one at step 2) and one at step 4), and that the former must be freed manually (it does not get freed by umount).

PARTTABLE_PARSER_SCRIPT="$SCRIPT_DIR/fdisk_parser.sh"
VDI_LOOP_SCRIPT="$SCRIPT_DIR/vdi_losetup.sh"
VDI_LOOP_DEVICE="$($VDI_LOOP_SCRIPT "$1")"
# This at one time hooks to a loop device and return the latter's name
OFFSET="$(fdisk -lu "$VDI_LOOP_DEVICE" 2>&1 | $PARTTABLE_PARSER_SCRIPT "$3")"
PARTITION_LOOP_DEVICE="$(losetup -v -f -o ${OFFSET} "${VDI_LOOP_DEVICE}"| sed -e "s/^${LOSETUP_MAGIC_STRING}//")"
echo "Virtual hard disk must be static and unshrinked to be accessed properly! This is decided at creation time."
echo "Virtual harddisk looped in ${VDI_LOOP_DEVICE}, partition of offset ${OFFSET} looped into ${PARTITION_LOOP_DEVICE}"
echo "type 'end' to dismount all and exit"
mount -o rw,noexec,noatime "${PARTITION_LOOP_DEVICE}" "$2"

ARG=""
until [ $ARG = end &> /dev/null ]
do
read ARG
done

umount "$2" || true
losetup -d "$PARTITION_LOOP_DEVICE" || true
losetup -d "$VDI_LOOP_DEVICE" || true

# Limitations: the loop device filename assigned by losetup shouldn't contain newlines or spaces to avoid interfering with fdisk output parsing, which identify fields through spaces and tidy table expectation (see remarks there). TODO: enable a preemptive check of this requirement (see remarks there as well).
# TODO: umount script
