#!/bin/bash
# M.B. Caminati
# Loops back a HD image (offsetly) residing inside a VDI image to a device file, whose name is printed to stdout
set -u 
set -e

SCRIPT_DIR="$(dirname "$(readlink -f "$0")")"

vdi_mbr_offset ()
{
	# Prints on stdout the offset in bytes of the hard disk master boot record inside a vdi file whose name is $1
echo "$1" > /dev/null
VDITOOL_APP="${SCRIPT_DIR}/vditool.sh"
MAGIC_STRING="offData="

RESULT="$($VDITOOL_APP DUMP "$1" |grep "$MAGIC_STRING")"
RESULT="${RESULT##*"$MAGIC_STRING"}"
# Todo: error handling
echo "$RESULT"
return 0
}

IMAGE_PATH="$(readlink -f "$1")"

# Check the vdi image isn't already in use by anyone else. This should be checked later by vditool as well.
#if lsof | grep --quiet "$IMAGE_PATH"
#then
 #exit 67
#fi

LOOP_DEVICE="`losetup -f`"
# There should be no unnecessary time loss between these two lines to avoid losing the free loopback device
losetup -o $(vdi_mbr_offset "$IMAGE_PATH") "$LOOP_DEVICE" "$IMAGE_PATH"
echo "$LOOP_DEVICE"
# out-of-reach enhancements: take control of how loop nodes are handled to make sure the one you get has a name suitable for later parsing, i.e. without spaces or weird chars.
# This is 99% already granted by losetup, so it's a bit paranoid. Anyway could be implemented by using mknod on a free minor number, but we would need to know a minor number (major is always 7 for loop devices) being free for sure, and I can't figure out a method to ascertain that, I'd need a list of all the minor numbers already occupied.
# The only approach I know of is to use ls -l showing major/minor numbers, but this implies one already nows which directory to ls, and so that's a dog eating its tail.

